//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/BeamDistributionItem.h
//! @brief     Defines class BeamDistributionItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_BEAMDISTRIBUTIONITEM_H
#define BORNAGAIN_GUI_MODELS_BEAMDISTRIBUTIONITEM_H

#include "GUI/Models/DistributionItems.h"
#include "Param/Distrib/ParameterDistribution.h"

//! The BeamDistributionItem handles wavelength, inclination and azimuthal parameter
//! distribution for BeamItem

class BA_CORE_API_ BeamDistributionItem : public SessionItem {
private:
    static const QString P_DISTRIBUTION;

public:
    explicit BeamDistributionItem(const QString& name, bool show_mean);

    std::unique_ptr<ParameterDistribution>
    getParameterDistributionForName(ParameterDistribution::WhichParameter which) const;

    virtual double meanValue() const;
    void resetToValue(double value);

    virtual double scaleFactor() const;

    SessionItem* distributionGroupItem() const;
    template <typename T> T* setDistributionGroupItem();
    SessionItem* distributionItem() const;

protected:
    void register_distribution_group(const QString& group_type);
    void initDistributionItem(bool show_mean);

    virtual std::unique_ptr<IDistribution1D> createDistribution1D() const;
};

template <typename T> T* BeamDistributionItem::setDistributionGroupItem()
{
    static_assert(std::is_base_of<DistributionItem, T>::value,
                  "Class must be derived from DistributionItem");

    return setGroupPropertyType<T>(P_DISTRIBUTION);
}


#endif // BORNAGAIN_GUI_MODELS_BEAMDISTRIBUTIONITEM_H
