//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/ComponentUtils.cpp
//! @brief     Implements ComponentUtils namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/ComponentUtils.h"
#include "GUI/Models/AxesItems.h"
#include "GUI/Models/MaterialDataItems.h"
#include "GUI/Models/VectorItem.h"

namespace {
QList<const SessionItem*> groupItems(const SessionItem& item);
}

const QStringList& GUI::Model::ComponentUtils::propertyRelatedTypes()
{
    static const QStringList result = {"Property",
                                       "GroupProperty",
                                       VectorItem::M_TYPE,
                                       BasicAxisItem::M_TYPE,
                                       AmplitudeAxisItem::M_TYPE,
                                       MaterialRefractiveDataItem::M_TYPE};
    return result;
}

QList<const SessionItem*> GUI::Model::ComponentUtils::componentItems(const SessionItem& item)
{
    static const QStringList& propertyRelated = GUI::Model::ComponentUtils::propertyRelatedTypes();

    QList<const SessionItem*> result;

    if (item.modelType() == "Property") {
        result.push_back(&item);

    } else if (item.modelType() == "GroupProperty") {
        result.push_back(&item);
        result += groupItems(item);

    } else {

        for (auto child : item.children()) {
            if (!child->isVisible())
                continue;

            if (propertyRelated.contains(child->modelType()))
                result.append(child);

            if (child->modelType() == "GroupProperty")
                result += groupItems(*child);
        }
    }

    return result;
}

namespace {
QList<const SessionItem*> groupItems(const SessionItem& item)
{
    ASSERT(item.modelType() == "GroupProperty");

    QList<const SessionItem*> result;
    for (auto grandchild : item.children()) {
        if (grandchild->isVisible())
            result += GUI::Model::ComponentUtils::componentItems(*grandchild);
    }

    return result;
}
} // namespace
