//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/FTDistributionItems.h
//! @brief     Defines FTDistribution1DItem's classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_FTDISTRIBUTIONITEMS_H
#define BORNAGAIN_GUI_MODELS_FTDISTRIBUTIONITEMS_H

#include "GUI/Models/SessionItem.h"
#include "Sample/Correlations/FTDistributions1D.h"
#include "Sample/Correlations/FTDistributions2D.h"

class BA_CORE_API_ FTDistribution1DItem : public SessionItem {
private:
    static const QString P_OMEGA;

public:
    virtual std::unique_ptr<IFTDistribution1D> createFTDistribution() const = 0;
    virtual ~FTDistribution1DItem() {}

    double omega() const;
    void setOmega(double omega);

protected:
    explicit FTDistribution1DItem(const QString& name);
    void add_omega_property();
};

class BA_CORE_API_ FTDistribution1DCauchyItem : public FTDistribution1DItem {
public:
    static const QString M_TYPE;

    FTDistribution1DCauchyItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution1DGaussItem : public FTDistribution1DItem {
public:
    static const QString M_TYPE;

    FTDistribution1DGaussItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution1DGateItem : public FTDistribution1DItem {
public:
    static const QString M_TYPE;

    FTDistribution1DGateItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution1DTriangleItem : public FTDistribution1DItem {
public:
    static const QString M_TYPE;

    FTDistribution1DTriangleItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution1DCosineItem : public FTDistribution1DItem {
public:
    static const QString M_TYPE;

    FTDistribution1DCosineItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution1DVoigtItem : public FTDistribution1DItem {
private:
    static const QString P_ETA;

public:
    static const QString M_TYPE;

    FTDistribution1DVoigtItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const;

    void setEta(double eta);
};

// --------------------------------------------------------------------------------------------- //

class BA_CORE_API_ FTDistribution2DItem : public SessionItem {
private:
    static const QString P_OMEGA_X;
    static const QString P_OMEGA_Y;
    static const QString P_GAMMA;

public:
    virtual std::unique_ptr<IFTDistribution2D> createFTDistribution() const = 0;

    double omegaX() const;
    void setOmegaX(double omega_x);
    double omegaY() const;
    void setOmegaY(double omega_y);

    double gamma() const;
    void setGamma(double gamma);

protected:
    explicit FTDistribution2DItem(const QString& name);
    void add_omega_properties();
    void add_gamma_property();
    void add_properties();
};

class BA_CORE_API_ FTDistribution2DCauchyItem : public FTDistribution2DItem {
public:
    static const QString M_TYPE;

    FTDistribution2DCauchyItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution2DGaussItem : public FTDistribution2DItem {
public:
    static const QString M_TYPE;

    FTDistribution2DGaussItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution2DGateItem : public FTDistribution2DItem {
public:
    static const QString M_TYPE;

    FTDistribution2DGateItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution2DConeItem : public FTDistribution2DItem {
public:
    static const QString M_TYPE;

    FTDistribution2DConeItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const;
};

class BA_CORE_API_ FTDistribution2DVoigtItem : public FTDistribution2DItem {
private:
    static const QString P_ETA;

public:
    static const QString M_TYPE;

    FTDistribution2DVoigtItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const;

    void setEta(double eta);
};

#endif // BORNAGAIN_GUI_MODELS_FTDISTRIBUTIONITEMS_H
