//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/FitParameterItems.h
//! @brief     Defines classes FitParameterItem, FitParameterLinkItem, FitParameterContainerItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_FITPARAMETERITEMS_H
#define BORNAGAIN_GUI_MODELS_FITPARAMETERITEMS_H

#include "Fit/Param/AttLimits.h"
#include "GUI/Models/SessionItem.h"

class FitParameterLinkItem;

//! FitParameterItems is a collection of items to define fit parameters in GUI.

//! The FitParameterItem class represents a fit parameter in GUI. Contains links to corresponding
//! ParameterItem's in a tuning tree.

class BA_CORE_API_ FitParameterItem : public SessionItem {
private:
    static const QString P_TYPE;
    static const QString P_START_VALUE;
    static const QString P_MIN;
    static const QString P_MAX;
    static const QString T_LINK;

public:
    static const QString M_TYPE;

    FitParameterItem();

    void initMinMaxValues(const RealLimits& limits);

    AttLimits attLimits() const;

    bool isValid() const;

    double startValue() const;
    void setStartValue(double start_value);
    SessionItem* startValueItem() const;

    double minimum() const;
    void setMinimum(double minimum);
    SessionItem* minimumItem() const;

    double maximum() const;
    void setMaximum(double maximum);
    SessionItem* maximumItem() const;

    QVector<FitParameterLinkItem*> linkItems() const;

    SessionItem* typeItem() const;

private:
    QString parameterType() const;
    void onTypeChange();
    void setLimitEnabled(const QString& name, bool enabled);
    bool isLimited() const;
    bool isFree() const;
    bool isLowerLimited() const;
    bool isUpperLimited() const;
    bool isFixed() const;
};

//! The FitParameterLinkItem class holds a link to ParameterItem in tuning tree.

class BA_CORE_API_ FitParameterLinkItem : public SessionItem {
private:
    static const QString P_LINK;

public:
    static const QString M_TYPE;

    FitParameterLinkItem();

    QString link() const;
    void setLink(const QString& link);
    SessionItem* linkItem() const;
};

//! The FitParameterContainerItem class is a collection of all defined fit parameters in JobItem.

namespace mumufit {
class Parameters;
}

class BA_CORE_API_ FitParameterContainerItem : public SessionItem {
private:
    static const QString T_FIT_PARAMETERS;

public:
    static const QString M_TYPE;

    FitParameterContainerItem();
    FitParameterItem* fitParameterItem(const QString& link);
    QVector<FitParameterItem*> fitParameterItems();
    bool isEmpty();
    void setValuesInParameterContainer(const QVector<double>& values,
                                       class ParameterContainerItem* parameterContainer);
    mumufit::Parameters createParameters() const;
};

#endif // BORNAGAIN_GUI_MODELS_FITPARAMETERITEMS_H
