//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/FitSuiteItem.h
//! @brief     Defines class FitSuiteItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_FITSUITEITEM_H
#define BORNAGAIN_GUI_MODELS_FITSUITEITEM_H

#include "GUI/Models/SessionItem.h"

class FitParameterContainerItem;
class MinimizerContainerItem;

class BA_CORE_API_ FitSuiteItem : public SessionItem {
private:
    static const QString P_UPDATE_INTERVAL;
    static const QString P_ITERATION_COUNT;
    static const QString P_CHI2;
    static const QString T_FIT_PARAMETERS_CONTAINER;
    static const QString T_MINIMIZER;

public:
    static const QString M_TYPE;

    explicit FitSuiteItem();

    FitParameterContainerItem* createFitParametersContainer();
    FitParameterContainerItem* fitParameterContainerItem();

    MinimizerContainerItem* createMinimizerContainer();
    MinimizerContainerItem* minimizerContainerItem();

    int updateInterval() const;
    void setUpdateInterval(int interval);
    static bool isUpdateIntervalPropertyName(const QString& name);

    int iterationCount() const;
    void setIterationCount(int count);
    static bool isIterationCountPropertyName(const QString& name);

    double chi2() const;
    void setChi2(double chi2);
};

#endif // BORNAGAIN_GUI_MODELS_FITSUITEITEM_H
