//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/Lattice2DItems.h
//! @brief     Defines classes Lattice2DItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_LATTICE2DITEMS_H
#define BORNAGAIN_GUI_MODELS_LATTICE2DITEMS_H

#include "GUI/Models/SessionItem.h"
class Lattice2D;

class BA_CORE_API_ Lattice2DItem : public SessionItem {
private:
    static const QString P_LATTICE_ROTATION_ANGLE;

protected:
    explicit Lattice2DItem(const QString& modelType);

public:
    virtual std::unique_ptr<Lattice2D> createLattice() const = 0;
    double unitCellArea() const;

    SessionItem* latticeRotationAngleItem() const;
    double latticeRotationAngle() const;
    void setLatticeRotationAngle(double angle);
};

class BA_CORE_API_ BasicLattice2DItem : public Lattice2DItem {
private:
    static const QString P_LATTICE_LENGTH1;
    static const QString P_LATTICE_LENGTH2;
    static const QString P_LATTICE_ANGLE;

public:
    static const QString M_TYPE;

    BasicLattice2DItem();
    std::unique_ptr<Lattice2D> createLattice() const;

    double latticeLength1() const;
    void setLatticeLength1(double length1);

    double latticeLength2() const;
    void setLatticeLength2(double length2);

    double latticeAngle() const;
    void setLatticeAngle(double angle);
};

class BA_CORE_API_ SquareLattice2DItem : public Lattice2DItem {
private:
    static const QString P_LATTICE_LENGTH;

public:
    static const QString M_TYPE;

    SquareLattice2DItem();
    std::unique_ptr<Lattice2D> createLattice() const;

    void setLatticeLength(double length);
};

class BA_CORE_API_ HexagonalLattice2DItem : public Lattice2DItem {
private:
    static const QString P_LATTICE_LENGTH;

public:
    static const QString M_TYPE;

    HexagonalLattice2DItem();
    std::unique_ptr<Lattice2D> createLattice() const;

    void setLatticeLength(double length);
};

#endif // BORNAGAIN_GUI_MODELS_LATTICE2DITEMS_H
