//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/LayerRoughnessItems.h
//! @brief     Defines classes LayerRoughnessItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_LAYERROUGHNESSITEMS_H
#define BORNAGAIN_GUI_MODELS_LAYERROUGHNESSITEMS_H

#include "GUI/Models/SessionItem.h"

class BA_CORE_API_ LayerZeroRoughnessItem : public SessionItem {
public:
    static const QString M_TYPE;

    LayerZeroRoughnessItem();
};

class BA_CORE_API_ LayerBasicRoughnessItem : public SessionItem {
private:
    static const QString P_SIGMA;
    static const QString P_HURST;
    static const QString P_LATERAL_CORR_LENGTH;

public:
    static const QString M_TYPE;

    LayerBasicRoughnessItem();

    double sigmaValue() const;
    void setSigmaValue(double sigma);

    double hurstParameterValue() const;
    void setHurstParameterValue(double hurst);

    double lateralCorrLengthValue() const;
    void setLateralCorrLengthValue(double lat_corr_length);
};

#endif // BORNAGAIN_GUI_MODELS_LAYERROUGHNESSITEMS_H
