//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/MaterialItem.h
//! @brief     Defines class MaterialItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_MATERIALITEM_H
#define BORNAGAIN_GUI_MODELS_MATERIALITEM_H

#include "Base/Types/Complex.h"
#include "Base/Vector/Vectors3D.h"
#include "GUI/Models/SessionItem.h"

class Material;

class BA_CORE_API_ MaterialItem : public SessionItem {
private:
    static const QString P_COLOR;
    static const QString P_MATERIAL_DATA;
    static const QString P_MAGNETIZATION;
    static const QString P_IDENTIFIER;

public:
    static const QString M_TYPE;

    MaterialItem();

    /// set refractive index as in 1 - delta + i * beta
    void setRefractiveIndex(const double delta, const double beta);
    void setScatteringLengthDensity(const complex_t index);
    /// \return true iff refractive index was given, otherwise SLD was given
    bool hasRefractiveIndex() const;
    /// \pre hasRefractiveIndex
    double refractiveIndexDelta() const;
    /// \pre hasRefractiveIndex
    double refractiveIndexBeta() const;
    /// \pre ! hasRefractiveIndex
    complex_t scatteringLengthDensity() const;

    QString identifier() const;
    void setIdentifier(const QString& id);

    QColor color() const;
    void setColor(const QColor& color);

    void setMagnetization(const kvector_t& magnetization);
    void hideMagnetization();

    std::unique_ptr<Material> createMaterial() const;
};

#endif // BORNAGAIN_GUI_MODELS_MATERIALITEM_H
