//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/MaterialItemUtils.h
//! @brief     Defines namespace GUI::Model::MaterialItemUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_MATERIALITEMUTILS_H
#define BORNAGAIN_GUI_MODELS_MATERIALITEMUTILS_H

#include "GUI/Models/ExternalProperty.h"
#include "GUI/Models/MaterialItem.h"
#include <QColor>
#include <QString>
#include <memory>

class Material;
class MaterialItemContainer;

namespace GUI::Model::MaterialItemUtils {

QColor suggestMaterialColor(const QString& name);
QColor randomMaterialColor();

ExternalProperty defaultMaterialProperty();

std::unique_ptr<Material> createDomainMaterial(const ExternalProperty& material_property);
std::unique_ptr<Material> createDomainMaterial(const ExternalProperty& material_property,
                                               const MaterialItemContainer& container);
MaterialItem* findMaterial(const ExternalProperty& material_property);

QString materialTag(const SessionItem& item);
QStringList materialRelatedModelTypes();

//! Constructs material property corresponding to given material.
ExternalProperty materialProperty(const MaterialItem& materialItem);

//! Constructs color property from given color.
ExternalProperty colorProperty(const QColor& color);

//! Gather material property items from a given item
QVector<SessionItem*> materialPropertyItems(SessionItem* item);

} // namespace GUI::Model::MaterialItemUtils

#endif // BORNAGAIN_GUI_MODELS_MATERIALITEMUTILS_H
