//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/RealLimitsItems.h
//! @brief     Defines RealLimitsItems's classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_REALLIMITSITEMS_H
#define BORNAGAIN_GUI_MODELS_REALLIMITSITEMS_H

#include <optional>

#include "GUI/Models/SessionItem.h"

class BA_CORE_API_ RealLimitsItem : public SessionItem {
private:
    static const QString P_XMIN;
    static const QString P_XMAX;

protected:
    RealLimitsItem(const QString& name, const std::optional<double>& lower = std::nullopt,
                   const std::optional<double>& upper = std::nullopt);
    double lowerLimit() const;
    void setLowerLimit(double lower);
    double upperLimit() const;
    void setUpperLimit(double upper);

public:
    virtual ~RealLimitsItem() {}
    virtual RealLimits createRealLimits(double scale_factor = 1.0) const = 0;
};

class BA_CORE_API_ LimitlessItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    LimitlessItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
};

class BA_CORE_API_ PositiveItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    PositiveItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
};

class BA_CORE_API_ NonnegativeItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    NonnegativeItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
};

class BA_CORE_API_ LowerLimitedItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    LowerLimitedItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
    using RealLimitsItem::setLowerLimit;
};

class BA_CORE_API_ UpperLimitedItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    UpperLimitedItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
    using RealLimitsItem::setUpperLimit;
};

class BA_CORE_API_ LimitedItem : public RealLimitsItem {
public:
    static const QString M_TYPE;

    LimitedItem();
    RealLimits createRealLimits(double scale_factor = 1.0) const;
    using RealLimitsItem::setLowerLimit;
    using RealLimitsItem::setUpperLimit;
};

#endif // BORNAGAIN_GUI_MODELS_REALLIMITSITEMS_H
