//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/RectangularDetectorItem.h
//! @brief     Defines class RectangularDetectorItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_RECTANGULARDETECTORITEM_H
#define BORNAGAIN_GUI_MODELS_RECTANGULARDETECTORITEM_H

#include "Base/Vector/Vectors3D.h"
#include "GUI/Models/ComboProperty.h"
#include "GUI/Models/DetectorItems.h"

class BasicAxisItem;

class BA_CORE_API_ RectangularDetectorItem : public DetectorItem {
private:
    static const QString P_X_AXIS;
    static const QString P_Y_AXIS;
    static const QString P_ALIGNMENT;
    static const QString P_NORMAL;
    static const QString P_DIRECTION;
    static const QString P_U0;
    static const QString P_V0;
    static const QString P_DBEAM_U0;
    static const QString P_DBEAM_V0;
    static const QString P_DISTANCE;

public:
    static const QString M_TYPE;

    RectangularDetectorItem();

    void setDetectorAlignment(const QString& alignment);

    int xSize() const override;
    int ySize() const override;
    void setXSize(int nx) override;
    void setYSize(int ny) override;

    const BasicAxisItem* xAxisItem() const;
    BasicAxisItem* xAxisItem();
    const BasicAxisItem* yAxisItem() const;
    BasicAxisItem* yAxisItem();

    static bool isAlignmentPropertyName(const QString& name);
    SessionItem* alignmentItem() const;
    ComboProperty alignment() const;
    void setAlignment(const ComboProperty& alignment);

    SessionItem* normalVectorItem() const;
    void setNormalVector(const kvector_t& normal);

    SessionItem* directionVectorItem() const;
    void setDirectionVector(const kvector_t& direction);

    SessionItem* u0Item() const;
    void setU0(double u0);

    SessionItem* v0Item() const;
    void setV0(double v0);

    SessionItem* directBeamU0Item() const;
    void setDirectBeamU0(double dbeam_u0);

    SessionItem* directBeamV0Item() const;
    void setDirectBeamV0(double dbeam_v0);

    SessionItem* distanceItem() const;
    void setDistance(double distance);

private:
    std::unique_ptr<IDetector2D> createDomainDetector() const override;
    void update_properties_appearance();
    kvector_t normalVector() const;
    kvector_t directionVector() const;

    bool m_is_constructed;
};

#endif // BORNAGAIN_GUI_MODELS_RECTANGULARDETECTORITEM_H
