//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/TransformToDomain.h
//! @brief     Defines namespace GUI::Transform::ToDomain
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODELS_TRANSFORMTODOMAIN_H
#define BORNAGAIN_GUI_MODELS_TRANSFORMTODOMAIN_H

#include "Device/Instrument/Instrument.h"
#include "Param/Distrib/Distributions.h" // for IDistribution1D
#include "Param/Distrib/ParameterDistribution.h"
#include "Sample/Aggregate/IInterferenceFunction.h"
#include "Sample/Aggregate/ParticleLayout.h"
#include "Sample/Multilayer/Layer.h"
#include "Sample/Multilayer/MultiLayer.h"
#include "Sample/Particle/IParticle.h"
#include "Sample/Slice/LayerRoughness.h"
#include <memory>

class AngularSpecScan;
class BeamDistributionItem;
class BeamItem;
class GISASSimulation;
class LayerItem;
class Material;
class MaterialItemContainer;
class SessionItem;
class ISimulation;

namespace GUI::Transform::ToDomain {

std::unique_ptr<Material> createDomainMaterial(const SessionItem& item);
std::unique_ptr<IParticle> createIParticle(const SessionItem& item);
std::unique_ptr<Layer> createLayer(const LayerItem& item);
std::unique_ptr<LayerRoughness> createLayerRoughness(const SessionItem& item);
std::unique_ptr<MultiLayer> createMultiLayer(const SessionItem& item);
std::unique_ptr<ParticleLayout> createParticleLayout(const SessionItem& item);

void addDistributionParametersToSimulation(const BeamItem& beam_item, GISASSimulation& simulation);
void addBeamDivergencesToScan(const BeamItem& beam_item, AngularSpecScan& simulation);

void setBeamDistribution(ParameterDistribution::WhichParameter which,
                         const BeamDistributionItem& item, ISimulation& simulation);

void setSimulationOptions(ISimulation* simulation, const SessionItem& item);
void setTransformationInfo(IParticle* result, const SessionItem& item);
void setPositionInfo(IParticle* result, const SessionItem& item);
void setRotationInfo(IParticle* result, const SessionItem& item);

} // namespace GUI::Transform::ToDomain

#endif // BORNAGAIN_GUI_MODELS_TRANSFORMTODOMAIN_H
