//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/ImportDataWidgets/CsvImportAssistant/CsvDataColumn.h
//! @brief     Defines classes CsvIntensityColumn and CsvCoordinateColumn
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_IMPORTDATAWIDGETS_CSVIMPORTASSISTANT_CSVDATACOLUMN_H
#define BORNAGAIN_GUI_VIEWS_IMPORTDATAWIDGETS_CSVIMPORTASSISTANT_CSVDATACOLUMN_H

#include "Device/Coord/Axes.h"
#include "GUI/Views/ImportDataWidgets/CsvImportAssistant/CsvNamespace.h"

class CsvIntensityColumn {
public:
    // Constructors:
    CsvIntensityColumn();
    CsvIntensityColumn(const CsvIntensityColumn& toCopy);
    CsvIntensityColumn(int colNum, double multiplier, GUI::View::csv::DataColumn values);

    // Getters:
    int columnNumber() const;
    double multiplier() const;
    GUI::View::csv::DataColumn values() const;

    // Setters:
    void setColNum(int const colNum);
    void setMultiplier(double const multiplier);
    void setValues(GUI::View::csv::DataColumn const values);
    void resetColumn(int colNum = -1, double multiplier = 1.,
                     GUI::View::csv::DataColumn values = {});

private:
    int m_colNum;
    double m_multiplier;
    GUI::View::csv::DataColumn m_values;
};

class CsvCoordinateColumn : public CsvIntensityColumn {
public:
    // Constructors:
    CsvCoordinateColumn();
    CsvCoordinateColumn(const CsvCoordinateColumn& toCopy);
    CsvCoordinateColumn(int colNum, double multiplier, GUI::View::csv::DataColumn values,
                        Axes::Coords units);

    // Getters:
    Axes::Coords units() const;
    GUI::View::csv::ColumnType name() const { return m_name; }

    // Setters:
    void setName(GUI::View::csv::ColumnType const name);
    void resetColumn(int colNum = -1, double multiplier = 1.,
                     GUI::View::csv::DataColumn values = {},
                     Axes::Coords units = Axes::Coords::NBINS,
                     GUI::View::csv::ColumnType name = GUI::View::csv::_intensity_);

private:
    Axes::Coords m_coords;
    GUI::View::csv::ColumnType m_name;
};

#endif // BORNAGAIN_GUI_VIEWS_IMPORTDATAWIDGETS_CSVIMPORTASSISTANT_CSVDATACOLUMN_H
