//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DetectorPresenter.cpp
//! @brief     Implements class DetectorPresenter
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/DetectorPresenter.h"
#include "GUI/Models/Error.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Models/SphericalDetectorItem.h"
#include "GUI/Views/InstrumentWidgets/RectangularDetectorEditor.h"
#include "GUI/Views/InstrumentWidgets/SphericalDetectorEditor.h"

namespace {
const QString SphericalDetectorPresentation = "Spherical";
const QString RectangularDetectorPresentation = "Rectangular";
} // namespace

DetectorPresenter::DetectorPresenter(QWidget* parent) : ItemComboWidget(parent)
{
    registerWidget(SphericalDetectorPresentation, create_new<SphericalDetectorEditor>);
    registerWidget(RectangularDetectorPresentation, create_new<RectangularDetectorEditor>);
    setToolBarVisible(false);
}

QString DetectorPresenter::itemPresentation() const
{
    if (!currentItem())
        return {};

    if (currentItem()->modelType() == SphericalDetectorItem::M_TYPE)
        return SphericalDetectorPresentation;
    else if (currentItem()->modelType() == RectangularDetectorItem::M_TYPE)
        return RectangularDetectorPresentation;
    else
        throw Error("DetectorPresenter::itemPresentation() -> Error. Wrong item "
                    "type '"
                    + currentItem()->modelType() + "'");
}

QStringList DetectorPresenter::activePresentationList(SessionItem* item)
{
    Q_UNUSED(item);
    return QStringList() << SphericalDetectorPresentation << RectangularDetectorPresentation;
}
