//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/MaterialEditor/MaterialItemUtils.cpp
//! @brief     Implements class MaterialItemUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/MaterialEditor/MaterialEditorUtils.h"
#include "GUI/Models/MaterialItemUtils.h"
#include "GUI/Models/MaterialModel.h"
#include "GUI/Models/MaterialModelStore.h"
#include "GUI/Views/MaterialEditor/MaterialEditorDialog.h"
#include "GUI/mainwindow/mainwindow.h"
#include <QColorDialog>

ExternalProperty
GUI::View::MaterialEditorUtils::selectMaterialProperty(const ExternalProperty& previous)
{
    MaterialEditorDialog dialog(GUI::Model::MaterialModelStore::materialModel(),
                                MainWindow::instance());
    dialog.setMaterialProperty(previous);
    if (dialog.exec() == QDialog::Accepted) {
        return dialog.selectedMaterialProperty();
    }

    return ExternalProperty();
}

ExternalProperty
GUI::View::MaterialEditorUtils::selectColorProperty(const ExternalProperty& previous)
{
    ExternalProperty result;

#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
    auto oldColor = previous.color();
    auto newColor = QColorDialog::getColor(oldColor);
    if (oldColor != newColor)
        result = GUI::Model::MaterialItemUtils::colorProperty(newColor);
#else
    bool ok = false;
    QRgb oldRgba = previous.color().rgba();
    QRgb newRgba = QColorDialog::getRgba(oldRgba, &ok, nullptr);
    if (ok && newRgba != oldRgba)
        result = GUI::Model::MaterialItemUtils::colorProperty(QColor::fromRgba(newRgba));
#endif

    return result;
}
