//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/SampleViewFactory.cpp
//! @brief     Implements class SampleViewFactory
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/SampleDesigner/SampleViewFactory.h"
#include "GUI/Models/InterferenceFunctionItems.h"
#include "GUI/Models/LayerItem.h"
#include "GUI/Views/SampleDesigner/InterferenceFunctionViews.h"
#include "GUI/Views/SampleDesigner/LayerView.h"
#include "GUI/Views/SampleDesigner/MesoCrystalView.h"
#include "GUI/Views/SampleDesigner/MultiLayerView.h"
#include "GUI/Views/SampleDesigner/ParticleCompositionView.h"
#include "GUI/Views/SampleDesigner/ParticleCoreShellView.h"
#include "GUI/Views/SampleDesigner/ParticleLayoutView.h"
#include "GUI/Views/SampleDesigner/ParticleView.h"
#include "GUI/Views/SampleDesigner/TransformationView.h"

QStringList SampleViewFactory::m_valid_item_types =
    {"MultiLayer",
     LayerItem::M_TYPE,
     "ParticleLayout",
     "Particle",
     "Rotation",
     "ParticleCoreShell",
     "ParticleComposition",
     "MesoCrystal",
     InterferenceFunction1DLatticeItem::M_TYPE,
     InterferenceFunction2DLatticeItem::M_TYPE,
     InterferenceFunction2DParaCrystalItem::M_TYPE,
     InterferenceFunctionFinite2DLatticeItem::M_TYPE,
     InterferenceFunctionHardDiskItem::M_TYPE,
     InterferenceFunctionRadialParaCrystalItem::M_TYPE};

bool SampleViewFactory::isValidType(const QString& model_type)
{
    if (model_type.startsWith("FormFactor")) {
        return true;
    } else {
        return m_valid_item_types.contains(model_type);
    }
}

IView* SampleViewFactory::createSampleView(const QString& model_type)
{
    if (model_type == "MultiLayer") {
        return new MultiLayerView();
    } else if (model_type == LayerItem::M_TYPE) {
        return new LayerView();
    } else if (model_type == "ParticleLayout") {
        return new ParticleLayoutView();
    } else if (model_type == "Particle") {
        return new ParticleView();
    } else if (model_type == "Rotation") {
        return new TransformationView();
    } else if (model_type == "ParticleCoreShell") {
        return new ParticleCoreShellView();
    } else if (model_type == "ParticleComposition") {
        return new ParticleCompositionView();
    } else if (model_type == "MesoCrystal") {
        return new MesoCrystalView();
    } else if (model_type == InterferenceFunction1DLatticeItem::M_TYPE) {
        return new InterferenceFunction1DLatticeView();
    } else if (model_type == InterferenceFunction2DLatticeItem::M_TYPE) {
        return new InterferenceFunction2DLatticeView();
    } else if (model_type == InterferenceFunction2DParaCrystalItem::M_TYPE) {
        return new InterferenceFunction2DParaCrystalView();
    } else if (model_type == InterferenceFunctionFinite2DLatticeItem::M_TYPE) {
        return new InterferenceFunctionFinite2DLatticeView();
    } else if (model_type == InterferenceFunctionHardDiskItem::M_TYPE) {
        return new InterferenceFunctionHardDiskView();
    } else if (model_type == InterferenceFunctionRadialParaCrystalItem::M_TYPE) {
        return new InterferenceFunctionRadialParaCrystalView();
    }
    return 0;
}
