//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/mainwindow/ProjectManagerStore.cpp
//! @brief     Implements namespace ProjectManagerStore
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/mainwindow/ProjectManagerStore.h"
#include "GUI/Models/Error.h"

namespace {
ProjectManager* theProjectManager = nullptr;
}

namespace GUI::Project::Manager {

ProjectManager* projectManager()
{
    if (!theProjectManager)
        throw Error("GUI::Project::Manager::projectManager() -> Error. Attempt to access "
                    "non existing ProjectManager.");

    return theProjectManager;
}

void subscribe(ProjectManager* projectManager)
{
    if (theProjectManager != nullptr)
        throw Error("GUI::Project::Manager::subscribe() -> Error. Attempt to subscribe "
                    "ProjectManager twice.");

    theProjectManager = projectManager;
}

void unsubscribe(ProjectManager* projectManager)
{
    if (theProjectManager != projectManager)
        throw Error("GUI::Project::Manager::unsubscribe -> Error. Attempt to unsubscribe "
                    "ProjectManager before it was subscribed.");

    theProjectManager = nullptr;
}
} // namespace GUI::Project::Manager
