//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Param/Node/NodeUtils.cpp
//! @brief     Implements collection of utility functions for INode.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Param/Node/NodeUtils.h"
#include "Param/Node/INode.h"
#include <algorithm>
#include <functional>
#include <iterator>

std::vector<std::tuple<const INode*, int, const INode*>> NodeUtils::progenyPlus(const INode* node,
                                                                                int level)
{
    std::vector<std::tuple<const INode*, int, const INode*>> result;
    result.push_back({node, level, nullptr});
    for (const auto* child : node->getChildren()) {
        for (const auto& [subchild, sublevel, subparent] : progenyPlus(child, level + 1))
            result.push_back({subchild, sublevel, child});
    }
    return result;
}
