//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/ZLimits.h
//! @brief     Defines class ZLimits.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_SCATTERING_ZLIMITS_H
#define BORNAGAIN_SAMPLE_SCATTERING_ZLIMITS_H

#include <iostream>
#include <limits>

//! Class that contains upper and lower limits of the z-coordinate for the slicing of
//! form factors.
//!
//! @ingroup intern

class ZLimits {
public:
    ZLimits();
    ZLimits(double _min, double _max);

    bool isFinite() const;

    double zBottom() const { return m_zmin; }
    double zTop() const { return m_zmax; }

    static constexpr double inf = std::numeric_limits<double>::infinity();

    static ZLimits enclosingInterval(const ZLimits& left, const ZLimits& right);

private:
    double m_zmin;
    double m_zmax;
};

bool operator==(const ZLimits& left, const ZLimits& right);
bool operator!=(const ZLimits& left, const ZLimits& right);

std::ostream& operator<<(std::ostream& ostr, const ZLimits& limits);

#endif // BORNAGAIN_SAMPLE_SCATTERING_ZLIMITS_H
#endif // USER_API
