//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Slice/SliceStack.h
//! @brief     Implements class SliceStack.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Slice/SliceStack.h"
#include "Sample/Slice/Slice.h"

void SliceStack::addSlice(double thickness, const Material& material,
                          const LayerRoughness* roughness)
{
    if (roughness)
        this->emplace_back(Slice(thickness, material, *roughness));
    else
        this->emplace_back(Slice(thickness, material));
}

//! Adds n times the same slice to the stack.

void SliceStack::addNSlices(size_t n, double thickness, const Material& material,
                            const LayerRoughness* roughness)
{
    if (thickness <= 0.0)
        return;
    if (n == 0)
        throw std::runtime_error("addNSlices: number of slices should be bigger than zero.");
    const double slice_thickness = thickness / n;
    addSlice(slice_thickness, material, roughness);
    for (size_t i = 1; i < n; ++i)
        addSlice(slice_thickness, material);
}


bool SliceStack::containsMagneticMaterial() const
{
    return std::any_of(this->cbegin(), this->cend(), [](const Slice& slice) -> bool {
        return slice.material().isMagneticMaterial();
    });
}
