//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Slice/SlicedFormFactorList.h
//! @brief     Defines class SlicedFormFactorList.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_SAMPLE_SLICE_SLICEDFORMFACTORLIST_H
#define BORNAGAIN_SAMPLE_SLICE_SLICEDFORMFACTORLIST_H

#include <map>
#include <memory>
#include <vector>

struct HomogeneousRegion;
class IFormFactor;
class IParticle;
class MultiLayer;
class SliceStack;

//! Contains and owns a list of form factors and the index of their containing layer.
//!
//! This class also handles the slicing of form factors if they cross layer interfaces.
//!
//! @ingroup intern

class SlicedFormFactorList {
public:
    SlicedFormFactorList() = delete;
    SlicedFormFactorList(const IParticle& particle, const SliceStack& slices, double z_ref);
    SlicedFormFactorList(const SlicedFormFactorList& other) = delete;

    size_t size() const;

    std::pair<const IFormFactor*, size_t> operator[](size_t index) const;

    const std::map<size_t, std::vector<HomogeneousRegion>>& regionMap() const;

private:
    void addParticle(IParticle& particle, const SliceStack& slices, double z_ref);

    std::vector<std::pair<std::unique_ptr<IFormFactor>, size_t>> m_ff_list;
    std::map<size_t, std::vector<HomogeneousRegion>> m_region_map;
};

#endif // BORNAGAIN_SAMPLE_SLICE_SLICEDFORMFACTORLIST_H
#endif // USER_API
