#include "Base/Const/Units.h"
#include "Device/Detector/IDetector2D.h"
#include "Device/Resolution/ConvolutionDetectorResolution.h"
#include "Device/Resolution/ResolutionFunction2DGaussian.h"
#include "GUI/Models/ComboProperty.h"
#include "GUI/Models/InstrumentItems.h"
#include "GUI/Models/InstrumentModel.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Models/ResolutionFunctionItems.h"
#include "Tests/GTestWrapper/google_test.h"

class TestDetectorItems : public ::testing::Test {
};

TEST_F(TestDetectorItems, test_detectorAlignment)
{
    InstrumentModel model;
    auto detector = model.insertItem<RectangularDetectorItem>();

    ComboProperty alignment = detector->alignment();
    // generic has some more items visible
    alignment.setValue("Generic");
    detector->setAlignment(alignment);
    EXPECT_TRUE(detector->normalVectorItem()->isVisible());

    // should be disabled if we switch
    alignment.setValue("Perpendicular to reflected beam");
    detector->setAlignment(alignment);
    EXPECT_FALSE(detector->normalVectorItem()->isVisible());
}

TEST_F(TestDetectorItems, test_resolutionFunction)
{
    InstrumentModel model;
    auto instrument = model.insertItem<GISASInstrumentItem>();

    DetectorItem* detectorItem = instrument->detectorItem();

    detectorItem->setResolutionFunctionType<ResolutionFunction2DGaussianItem>();

    auto detector = detectorItem->createDetector();
    auto convol =
        dynamic_cast<const ConvolutionDetectorResolution*>(detector->detectorResolution());
    auto gaussian =
        dynamic_cast<const ResolutionFunction2DGaussian*>(convol->getResolutionFunction2D());

    EXPECT_EQ(Units::rad2deg(gaussian->getSigmaX()), 0.02);
}
