#include "GUI/Models/LayerRoughnessItems.h"
#include "GUI/Models/TransformFromDomain.h"
#include "GUI/Models/TransformToDomain.h"
#include "Tests/GTestWrapper/google_test.h"

class TestLayerRoughnessItems : public ::testing::Test {
};

TEST_F(TestLayerRoughnessItems, test_LayerRoughnessToDomain)
{
    LayerBasicRoughnessItem roughnessItem;
    roughnessItem.setSigmaValue(10.0);
    roughnessItem.setHurstParameterValue(20.0);
    roughnessItem.setLateralCorrLengthValue(30.0);

    auto P_roughness = GUI::Transform::ToDomain::createLayerRoughness(roughnessItem);
    EXPECT_EQ(P_roughness->getSigma(), roughnessItem.sigmaValue());
    EXPECT_EQ(P_roughness->getHurstParameter(), roughnessItem.hurstParameterValue());
    EXPECT_EQ(P_roughness->getLatteralCorrLength(), roughnessItem.lateralCorrLengthValue());

    LayerZeroRoughnessItem zeroRoughnessItem;
    EXPECT_TRUE(GUI::Transform::ToDomain::createLayerRoughness(zeroRoughnessItem) == nullptr);
}

TEST_F(TestLayerRoughnessItems, test_LayerRoughnessFromDomain)
{
    LayerRoughness roughness(10.0, 20.0, 30.0);
    LayerBasicRoughnessItem roughnessItem;
    GUI::Transform::FromDomain::setRoughnessItem(&roughnessItem, roughness);
    EXPECT_EQ(roughness.getSigma(), roughnessItem.sigmaValue());
    EXPECT_EQ(roughness.getHurstParameter(), roughnessItem.hurstParameterValue());
    EXPECT_EQ(roughness.getLatteralCorrLength(), roughnessItem.lateralCorrLengthValue());
}
