#include "GUI/Models/AxesItems.h"
#include "GUI/Models/InstrumentItems.h"
#include "GUI/Models/InstrumentModel.h"
#include "GUI/Models/IntensityDataItem.h"
#include "GUI/Models/JobItemUtils.h"
#include "GUI/Models/RealDataItem.h"
#include "GUI/Models/RealDataModel.h"
#include "GUI/Models/RectangularDetectorItem.h"
#include "GUI/Views/ImportDataWidgets/LinkInstrumentManager.h"
#include "Tests/GTestWrapper/google_test.h"
#include "Tests/UnitTests/GUI/Utils.h"
#include <QSignalSpy>
#include <QTest>

class TestLinkInstrument : public ::testing::Test {
};

TEST_F(TestLinkInstrument, test_canLinkToInstrument)
{
    InstrumentModel instrumentModel;
    RealDataModel realDataModel;
    LinkInstrumentManager manager;
    manager.setModels(&instrumentModel, &realDataModel);

    // populating instrument model
    auto instrument = instrumentModel.insertItem<GISASInstrumentItem>();
    const QString identifier = instrument->id();

    // populating real data model, setting intensity data
    RealDataItem* realData = GuiUnittestUtils::createRealData("RealData", realDataModel);
    GUI::Model::JobItemUtils::createDefaultDetectorMap(realData->dataItem(), instrument);

    ASSERT_TRUE(manager.canLinkDataToInstrument(realData, identifier, true));

    // making link
    realData->setInstrumentId(identifier);
    EXPECT_EQ(manager.linkedRealDataItems(instrument), QList<RealDataItem*>() << realData);

    // changing detector type and checking that link remain
    instrument->setDetectorGroup<RectangularDetectorItem>();
    EXPECT_EQ(manager.linkedRealDataItems(instrument), QList<RealDataItem*>() << realData);

    // changing detector binning and checking that link is destroyed
    RectangularDetectorItem* detectorItem =
        dynamic_cast<RectangularDetectorItem*>(instrument->detectorItem());
    auto x_axis = detectorItem->xAxisItem();
    x_axis->setBinCount(10);

    EXPECT_EQ(manager.linkedRealDataItems(instrument), QList<RealDataItem*>());
    EXPECT_EQ(realData->instrumentId(), QString());
}
