#include "GUI/Models/ComboProperty.h"
#include "GUI/Models/DocumentModel.h"
#include "GUI/Models/LayerItem.h"
#include "GUI/Models/MultiLayerItem.h"
#include "GUI/Models/ParticleCompositionItem.h"
#include "GUI/Models/ParticleItem.h"
#include "GUI/Models/ParticleLayoutItem.h"
#include "GUI/Models/SampleModel.h"
#include "GUI/Models/SessionItemUtils.h"
#include "GUI/Models/SimulationOptionsItem.h"
#include "Tests/GTestWrapper/google_test.h"
#include <QtTest>

using GUI::Session::ItemUtils::ParentRow;

class TestMapperCases : public ::testing::Test {
};

TEST_F(TestMapperCases, test_ParticeleCompositionUpdate)
{
    SampleModel model;
    auto multilayer = model.insertItem<MultiLayerItem>();
    auto layer = model.insertItem<LayerItem>(multilayer);
    auto layout = model.insertItem<ParticleLayoutItem>(layer);

    // composition added to layout should have abundance enabled
    auto compositionFree = model.insertItem<ParticleCompositionItem>(layout);
    EXPECT_TRUE(compositionFree->getItem(ParticleItem::P_ABUNDANCE)->isEnabled());
}

TEST_F(TestMapperCases, test_SimulationOptionsComputationToggle)
{
    DocumentModel model;
    model.insertItem<SimulationOptionsItem>();

    SimulationOptionsItem* item = model.simulationOptionsItem();

    EXPECT_EQ(item->getComputationMethod(), "Analytical");
    EXPECT_FALSE(item->monteCarloPointsItem()->isEnabled());

    item->setComputationMethod("Monte-Carlo Integration");
    EXPECT_TRUE(item->monteCarloPointsItem()->isEnabled());
}
